package com.sun.xml.internal.ws.encoding;

import javax.xml.transform.Transformer;
import java.io.OutputStreamWriter;
import javax.xml.transform.stream.StreamResult;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import javax.xml.transform.Source;
import java.io.OutputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import javax.activation.DataSource;
import java.util.Arrays;
import javax.activation.ActivationDataFlavor;
import javax.xml.transform.stream.StreamSource;
import java.awt.datatransfer.DataFlavor;
import javax.activation.DataContentHandler;

public class XmlDataContentHandler implements DataContentHandler
{
    private final DataFlavor[] flavors;
    
    public XmlDataContentHandler() throws ClassNotFoundException {
        (this.flavors = new DataFlavor[3])[0] = new ActivationDataFlavor(StreamSource.class, "text/xml", "XML");
        this.flavors[1] = new ActivationDataFlavor(StreamSource.class, "application/xml", "XML");
        this.flavors[2] = new ActivationDataFlavor(String.class, "text/xml", "XML String");
    }
    
    public DataFlavor[] getTransferDataFlavors() {
        return (DataFlavor[])Arrays.copyOf(this.flavors, this.flavors.length);
    }
    
    public Object getTransferData(DataFlavor df, DataSource ds) throws IOException {
        DataFlavor[] flavors = this.flavors;
        int length = flavors.length;
        int i = 0;
        while (i < length) {
            DataFlavor aFlavor = flavors[i];
            if (aFlavor.equals(df)) {
                return this.getContent(ds);
            }
            ++i;
        }
        return null;
    }
    
    public Object getContent(DataSource ds) throws IOException {
        String ctStr = ds.getContentType();
        String charset = null;
        if (ctStr != null) {
            ContentType ct = new ContentType(ctStr);
            if (!this.isXml(ct)) {
                throw new IOException("Cannot convert DataSource with content type \"" + ctStr + "\" to object in XmlDataContentHandler");
            }
            charset = ct.getParameter("charset");
        }
        return (charset != null) ? new StreamSource(new InputStreamReader(ds.getInputStream()), charset) : new StreamSource(ds.getInputStream());
    }
    
    public void writeTo(Object obj, String mimeType, OutputStream os) throws IOException {
        if (!(obj instanceof DataSource) && !(obj instanceof Source) && !(obj instanceof String)) {
            throw new IOException("Invalid Object type = " + obj.getClass() + ". XmlDataContentHandler can only convert DataSource|Source|String to XML.");
        }
        ContentType ct = new ContentType(mimeType);
        if (!this.isXml(ct)) {
            throw new IOException("Invalid content type \"" + mimeType + "\" for XmlDataContentHandler");
        }
        String charset = ct.getParameter("charset");
        if ((obj instanceof String)) {
            String s = (String)obj;
            if (charset == null) {
                charset = "utf-8";
            }
            OutputStreamWriter osw = new OutputStreamWriter(os, charset);
            osw.write(s, 0, s.length());
            osw.flush();
            return;
        }
        else {
            Source source = (obj instanceof DataSource) ? ((Source)this.getContent((DataSource)obj)) : ((Source)obj);
            try {
                Transformer transformer = XmlUtil.newTransformer();
                if (charset != null) {
                    transformer.setOutputProperty("encoding", charset);
                }
                StreamResult result = new StreamResult(os);
                transformer.transform(source, result);
            }
            catch (Exception ex) {
                throw new IOException("Unable to run the JAXP transformer in XmlDataContentHandler " + ex.getMessage());
            }
            return;
        }
    }
    
    private boolean isXml(ContentType ct) {
        return ct.getSubType().equals("xml") && (ct.getPrimaryType().equals("text") || ct.getPrimaryType().equals("application"));
    }
}
